body {{ 
  word-wrap: break-word;
}}
a {{
    color: #fff;
}}
.msg-user {{
    color: #bdbdbd !important;
    white-space: pre-wrap;
    width: 100%;
    max-width: 100%;
}}
.msg-bot {{
    color: #fff;
    white-space: pre-wrap;
    width: 100%;
    max-width: 100%;
}}
.cmd {{
    color: #4d4d4d;
    text-decoration: none;
}}
.ts {{
    color: #4d4d4d;
}}
.list {{
}}
pre {{
    color: {QTMATERIAL_PRIMARYLIGHTCOLOR};
    background-color: #202225;
    font-family: 'Space Mono';
    display: block;
    word-wrap: break-word;
    white-space: pre-wrap;
}}
code {{
    color: {QTMATERIAL_PRIMARYLIGHTCOLOR};
    word-wrap: break-word;
    font-family: 'Space Mono';
}}